
package eu.dnetlib.dhp.oa.graph.dump.filterentities;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.schema.common.ModelSupport;
import eu.dnetlib.dhp.schema.oaf.Result;
import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.util.Optional;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

/**
 * @author miriam.baglioni
 * @Date 20/03/24
 */
public class SelectCommunityEntities implements Serializable {
	private static final Logger log = LoggerFactory.getLogger(SelectCommunityEntities.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				FilterEntities.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/skgif/eosc_entities_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);

		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String inputPath = parser.get("sourcePath");
		log.info("inputPath: {}", inputPath);

		final String filterPath = parser.get("filterPath");
		log.info("filterPath: {}", filterPath);

		final String communityId = parser.get("communityId");
		log.info("communityId: {}", communityId);

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {

				selectEntities(spark, inputPath, filterPath, communityId);

			});
	}

	private static <R extends Result> void selectEntities(SparkSession spark, String inputPath, String filterPath, String communityId) {
		ModelSupport.entityTypes.keySet().forEach(e -> {
			if (ModelSupport.isResult(e)) {

				spark
					.read()
					.schema(Encoders.bean(Result.class).schema())
					.json(inputPath + e.name())
					.where("datainfo.deletedbyinference != true and datainfo.invisible != true")
					.select("id", "context")
					.where("array_contains(context.id,'"+communityId+"')")
					.drop("context")
					.distinct()
					.write()
					.mode(SaveMode.Overwrite)
					.option("compression", "gzip")
					.parquet(filterPath + e.name() + "_ids");

			}

		});
	}

}
